# Copyright (C) 2023, Hopsworks AB. All rights reserved

import os
import joblib
import numpy as np


class Predict(object):

    def __init__(self):
        model_local_dir = (
            os.environ["ARTIFACT_FILES_PATH"]
            if "ARTIFACT_FILES_PATH" in os.environ
            else "."
        )
        model_local_path = model_local_dir + "/iris_knn.pkl"

        if os.path.exists(model_local_path):
            print("Loading model file...")
            self.model = joblib.load(model_local_path)
        else:
            print("No model file found")

            class MirrorModel:
                def predict(self, inputs):
                    return np.array(inputs)

            self.model = MirrorModel()

        print("Initialization completed")

    def predict(self, inputs):
        """Serves a prediction request usign a trained model"""

        if isinstance(inputs, list):
            from hsml.client.istio.utils.infer_type import InferInput

            if isinstance(inputs[0], InferInput):  # List[InferInput]
                from kserve.utils.numpy_codec import from_np_dtype

                # reshape inputs
                reshaped_inputs = np.reshape(inputs[0].data, inputs[0].shape)

                # predict
                predictions = self.model.predict(reshaped_inputs)

                # return predictor outputs
                return [
                    {
                        "name": "output-0",
                        "shape": list(predictions.shape),
                        "datatype": from_np_dtype(predictions.dtype),
                        "data": predictions.flatten().tolist(),
                    }
                ]

        # Dict, List[list], List[float], List[int], ...
        return self.model.predict(inputs).tolist()

    def classify(self, inputs):
        """Serves a classification request using a trained model"""
        return "not implemented"

    def regress(self, inputs):
        """Serves a regression request using a trained model"""
        return "not implemented"
