# Copyright (C) 2023, Hopsworks AB. All rights reserved

import os
from sklearn.externals import joblib
import logging


class Predict(object):

    def __init__(self):
        logging.info("[Predictor] __init__()")
        # NOTE: env var ARTIFACT_FILES_PATH has the local path to the model artifact files
        self.model = joblib.load(os.environ["ARTIFACT_FILES_PATH"] + "/iris_knn.pkl")

    def predict(self, inputs):
        """ Serves a prediction request from a trained model"""

        if isinstance(inputs, dict):  # Dict
            return self.model.predict(inputs).tolist()
        elif isinstance(inputs, list):
            if isinstance(inputs[0], list):  # List[List]
                return self.model.predict(inputs).tolist()
            else:  # List[InferInput]
                import numpy as np
                from kserve.utils.numpy_codec import from_np_dtype

                # reshape inputs
                reshaped_inputs = np.reshape(inputs[0].data, inputs[0].shape)

                # predict
                predictions = self.model.predict(reshaped_inputs)

                # return predictor outputs
                return [{
                    "name": "output-0",
                    "shape": list(predictions.shape),
                    "datatype": from_np_dtype(predictions.dtype),
                    "data": predictions.flatten().tolist(),
                }]
