# Copyright (C) 2024, Hopsworks AB. All rights reserved

import logging
import numpy as np
from kserve.utils.numpy_codec import from_np_dtype
from hsml.client.istio.utils.infer_type import (
    InferInput,
    InferOutput,
)


class Transformer:

    def __init__(self):
        logging.info("[Transformer] __init__(): user-provided init")

    def preprocess(self, inputs):
        logging.info("[Transformer] preprocess()")

        if isinstance(inputs, list):
            if isinstance(inputs[0], InferInput):  # List[InferInput]

                # reshape inputs
                reshaped_inputs = np.reshape(inputs[0].data, inputs[0].shape)

                # process the inputs
                processed_inputs = reshaped_inputs

                # build predictor inputs
                return [
                    {
                        "name": "input-0",
                        "shape": list(processed_inputs.shape),
                        "datatype": from_np_dtype(processed_inputs.dtype),
                        "data": processed_inputs.flatten().tolist(),
                    }
                ]

        # Dict, List[List], List[float]...
        return inputs

    def postprocess(self, outputs):
        logging.info("[Transformer] postprocess()")

        if isinstance(outputs, list):
            if isinstance(outputs[0], InferOutput):  # List[InferOutput]

                # reshape outputs
                reshaped_outputs = np.reshape(outputs[0].data, outputs[0].shape)

                # process the outputs
                processed_outputs = reshaped_outputs

                # build predictor outputs
                return [
                    {
                        "name": "output-0",
                        "shape": list(processed_outputs.shape),
                        "datatype": from_np_dtype(processed_outputs.dtype),
                        "data": processed_outputs.flatten().tolist(),
                    }
                ]

        # Dict, List[List], List[float]...
        return outputs
