=begin
 Copyright (C) 2022, Hopsworks AB. All rights reserved
=end
require 'pp'

describe "On #{ENV['OS']}" do
  before :all do
    $stdout.sync = true
    @debugOpt = false
    @cleanUp = true
  end
  after :all do
    clean_all_test_projects(spec: "ee_explicit_provenance") if @cleanUp
  end

  context "multi project" do
    before :all do
      setup_main_project
      setup_shared_project
      setup_unshared_project
    end

    def setup_main_project()
      @user1_params = {email: "user1_#{random_id}@email.com", first_name: "User", last_name: "1", password: "Pass123"}
      @user1 = create_user_with_role(@user1_params, "HOPS_USER")
      pp "user email: #{@user1[:email]}" if defined?(@debugOpt) && @debugOpt

      create_session(@user1[:email], @user1_params[:password])
      @project1 = create_project
      pp @project1[:projectname] if defined?(@debugOpt) && @debugOpt

      @connector1 = create_jdbc_connector_checked(@project1[:id])
      @on_demand_fg1 = create_on_demand_featuregroup_checked(@project1[:id], @connector1["id"], name: "on_demand_fg1")
      @cached_fg1 = create_cached_featuregroup_checked2(@project1[:id], name: "cached_fg1")
      @derived_fg1 = create_cached_featuregroup_checked2(@project1[:id], name: "derived_fg1", parents: [@cached_fg1, @on_demand_fg1])
      @feature_view1 = create_feature_view_checked(@project1[:id], fg: @derived_fg1, name: "feature_view1")
      @training_dataset1 = create_featureview_training_dataset_checked(@project1, @feature_view1)
    end

    def setup_shared_project
      @user2_params = {email: "user2_#{random_id}@email.com", first_name: "User", last_name: "2", password: "Pass123"}
      @user2 = create_user_with_role(@user2_params, "HOPS_USER")
      pp "user email: #{@user2[:email]}" if defined?(@debugOpt) && @debugOpt

      create_session(@user2[:email], @user2_params[:password])
      @project2 = create_project
      pp @project2[:projectname] if defined?(@debugOpt) && @debugOpt

      create_session(@user1_params[:email], @user1_params[:password])
      share_dataset_checked(@project1, "#{@project1[:projectname].downcase}_featurestore.db", @project2[:projectname], datasetType: "FEATURESTORE")
      share_dataset_checked(@project1, "Models", @project2[:projectname], datasetType: "DATASET")

      create_session(@user2_params[:email], @user2_params[:password])
      accept_dataset_checked(@project2, "#{@project1[:projectname]}::#{@project1[:projectname].downcase}_featurestore.db", datasetType: "FEATURESTORE")
      accept_dataset_checked(@project2, "#{@project1[:projectname]}::Models", datasetType: "DATASET")

      @shared_derived_fg1 = create_cached_featuregroup_checked2(@project2[:id], name: "shared_derived_fg1", parents: [@cached_fg1, @on_demand_fg1])
      @shared_feature_view1 = create_feature_view_checked(@project2[:id], fg: @derived_fg1, name: "shared_feature_view1")
    end

    def setup_unshared_project
      @user3_params = {email: "user3_#{random_id}@email.com", first_name: "User", last_name: "3", password: "Pass123"}
      @user3 = create_user_with_role(@user3_params, "HOPS_USER")
      pp "user email: #{@user3[:email]}" if defined?(@debugOpt) && @debugOpt

      create_session(@user3[:email], @user3_params[:password])
      @project3 = create_project
      pp @project3[:projectname] if defined?(@debugOpt) && @debugOpt

      create_session(@user1_params[:email], @user1_params[:password])
      share_dataset_checked(@project1, "#{@project1[:projectname].downcase}_featurestore.db", @project3[:projectname], datasetType: "FEATURESTORE")
      share_dataset_checked(@project1, "Models", @project3[:projectname], datasetType: "DATASET")

      create_session(@user3_params[:email], @user3_params[:password])
      accept_dataset_checked(@project3, "#{@project1[:projectname]}::#{@project1[:projectname].downcase}_featurestore.db", datasetType: "FEATURESTORE")
      accept_dataset_checked(@project3, "#{@project1[:projectname]}::Models", datasetType: "DATASET")

      @unshared_derived_fg1 = create_cached_featuregroup_checked2(@project3[:id], name: "unshared_derived_fg1", parents: [@cached_fg1, @on_demand_fg1])
      @unshared_feature_view1 = create_feature_view_checked(@project3[:id], fg: @derived_fg1, name: "unshared_feature_view1")

      create_session(@user1_params[:email], @user1_params[:password])
      unshare_dataset_checked(@project1, "#{@project1[:projectname].downcase}_featurestore.db", @project3, dataset_type: "FEATURESTORE")
      unshare_dataset_checked(@project1, "Models", @project3, dataset_type: "DATASET")
    end

    context "same project" do
      it "check sc to fg explicit link" do
        create_session(@user1_params[:email], @user1_params[:password])
        links1 = get_storage_connector_links_obj(@project1[:id], @connector1)
        links2 = get_checked(URI.parse(links1["downstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        links3 = get_feature_group_links_obj(@project1[:id], @on_demand_fg1)
        links4 = get_checked(URI.parse(links3["upstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        expect(links1).to eq links4
        expect(links2).to eq links3
      end

      it "check fg to fg explicit link" do
        create_session(@user1_params[:email], @user1_params[:password])
        links1 = get_feature_group_links_obj(@project1[:id], @cached_fg1)
        links2 = get_checked(URI.parse(links1["downstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        links3 = get_feature_group_links_obj(@project1[:id], @derived_fg1)
        links4 = get_checked(URI.parse(links3["upstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        expect(links1).to eq links4
        expect(links2).to eq links3
      end

      it "check fg to fv explicit link" do
        create_session(@user1_params[:email], @user1_params[:password])
        links1 = get_feature_group_links_obj(@project1[:id], @derived_fg1)
        links2 = get_checked(URI.parse(links1["downstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        links3 = get_feature_view_links_obj(@project1[:id], @feature_view1)
        links4 = get_checked(URI.parse(links3["upstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        expect(links1).to eq links4
        expect(links2).to eq links3
      end

      it "check fv to td explicit link" do
        create_session(@user1_params[:email], @user1_params[:password])
        links1 = get_feature_view_links_obj(@project1[:id], @feature_view1)
        links2 = get_checked(URI.parse(links1["downstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        links3 = get_training_dataset_links_obj(@project1[:id], @feature_view1, @training_dataset1)
        links4 = get_checked(URI.parse(links3["upstream"][0]["href"]).path + "?expand=PROVENANCE_ARTIFACTS")
        expect(links1).to eq links4
        expect(links2).to eq links3
      end
    end

    context "shared project" do
      it "check fg to fg explicit link" do
        create_session(@user2_params[:email], @user2_params[:password])
        links1 = get_feature_group_links_obj(@project2[:id], @cached_fg1, feature_store_project_id: @project1[:id])
        expect(links1["downstream"][1]["node"]["artifact"]["name"]).to eq @shared_derived_fg1["name"]
        links2 = get_feature_group_links_obj(@project2[:id], @shared_derived_fg1)
        expect(links2["upstream"][0]["node"]["artifact"]["name"]).to eq @cached_fg1["name"]
        expect(links2["upstream"][0]["node"]["shared"]).to be true
        expect(links2["upstream"][0]["node"]["accessible"]).to be true
      end

      it "check fg to fv explicit link" do
        create_session(@user2_params[:email], @user2_params[:password])
        links1 = get_feature_group_links_obj(@project2[:id], @derived_fg1, feature_store_project_id: @project1[:id])
        expect(links1["downstream"][1]["node"]["artifact"]["name"]).to eq @shared_feature_view1["name"]
        links2 = get_feature_view_links_obj(@project2[:id], @shared_feature_view1)
        expect(links2["upstream"][0]["node"]["artifact"]["name"]).to eq @derived_fg1["name"]
        expect(links2["upstream"][0]["node"]["shared"]).to be true
        expect(links2["upstream"][0]["node"]["accessible"]).to be true
      end
    end

    context "unshared project" do
      it "check fg to fg explicit link" do
        create_session(@user3_params[:email], @user3_params[:password])
        links1 = get_feature_group_links_obj(@project3[:id], @unshared_derived_fg1)
        expect(links1["upstream"][0]["node"]["artifact"]["name"]).to eq @cached_fg1["name"]
        expect(links1["upstream"][0]["node"]["shared"]).to be true
        expect(links1["upstream"][0]["node"]["accessible"]).to be false
      end

      it "check fg to fv explicit link" do
        create_session(@user3_params[:email], @user3_params[:password])
        links1 = get_feature_view_links_obj(@project3[:id], @unshared_feature_view1)
        expect(links1["upstream"][0]["node"]["artifact"]["name"]).to eq @derived_fg1["name"]
        expect(links1["upstream"][0]["node"]["shared"]).to be true
        expect(links1["upstream"][0]["node"]["accessible"]).to be false
      end
    end
  end
end
