# This file is part of Hopsworks
# Copyright (C) 2024, Hopsworks AB. All rights reserved
#
# Hopsworks is free software: you can redistribute it and/or modify it under the terms of
# the GNU Affero General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# Hopsworks is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License along with this program.
# If not, see <https://www.gnu.org/licenses/>.
#

require 'json'

describe "On #{ENV['OS']}" do
  after(:all) { clean_all_test_projects(spec: "featureviewtransformation") }

  describe "feature group transformation" do
    describe "internal" do
      context 'with valid project, featurestore service enabled' do
        before :all do
          with_valid_project
        end

        it "should be able to attach transformation function." do
          featurestore_id = get_featurestore_id(@project.id)
          featuregroup_suffix = short_random_id

          # create transformation function
          json_result = register_transformation_fn(@project.id, featurestore_id)
          transformation_function_feature_a = JSON.parse(json_result)
          expect_status_details(200)

          feature_schema = [
            {type: "INT", name: "a_testfeature1", featureGroupFeatureName: "a_testfeature1", label: false},
            {type: "INT", name: "b_testfeature1", featureGroupFeatureName: "b_testfeature1", label: false}
          ]
          transformation_function_feature_a["hopsworksUdf"]["transformationFeatures"] = ["a_testfeature1"]
          transformation_function_feature_a["hopsworksUdf"]["outputTypes"] = ["double"]

          transformation_feature_b = {
            id: nil,
            version: 1,
            featurestoreId: 11,
            hopsworksUdf: {
              sourceCode: "\\n@hopworks.udf(int)\\ndef plus_two(data1 : pd.Series):\\n    return data1 + 2",
              name: "plus_two",
              outputTypes: ["bigint"],
              transformationFeatures: ["b_testfeature1"],
              transformationFunctionArgumentNames: ["data1"],
            }
          }

          project = get_project
          featurestore_id = get_featurestore_id(project.id)
          features = [
            {
                type: "INT",
                name: "plus_one",
                description: "--",
                primary: false,
                onDemand: true
            },
            {
                type: "INT",
                name: "plus_two",
                description: "--",
                primary: false,
                onDemand: true
            }
          ]
          json_result, featuregroup_name = create_cached_featuregroup(project.id, featurestore_id, features:features, online:true, transformation_functions: [transformation_function_feature_a, transformation_feature_b])

          parsed_json = JSON.parse(json_result)
          expect_status_details(201)
          
          feature_group_name = parsed_json["name"]
          feature_group_version = parsed_json["version"]

          feature_transformation = parsed_json["transformationFunctions"].select{ |f| f["hopsworksUdf"]["name"] == "plus_one"}.first
          expect(feature_transformation["hopsworksUdf"]["name"]).to eql("plus_one")
          expect(feature_transformation["hopsworksUdf"]["outputTypes"]).to eql(["double"])
          expect(feature_transformation["hopsworksUdf"]["transformationFeatures"]).to eql(["a_testfeature1"])
          expect(feature_transformation["hopsworksUdf"]["transformationFunctionArgumentNames"]).to eql(["data1"])
          expect(feature_transformation["hopsworksUdf"]["droppedArgumentNames"]).to eql(["data1"])
          expect(feature_transformation["hopsworksUdf"]["statisticsArgumentNames"]).to eql(nil)

          feature_transformation = parsed_json["transformationFunctions"].select{ |f| f["hopsworksUdf"]["name"] == "plus_two"}.first
          expect(feature_transformation["hopsworksUdf"]["name"]).to eql("plus_two")
          expect(feature_transformation["hopsworksUdf"]["outputTypes"]).to eql(["bigint"])
          expect(feature_transformation["hopsworksUdf"]["transformationFeatures"]).to eql(["b_testfeature1"])
          expect(feature_transformation["hopsworksUdf"]["transformationFunctionArgumentNames"]).to eql(["data1"])
          expect(feature_transformation["hopsworksUdf"]["droppedArgumentNames"]).to eql(nil)
          expect(feature_transformation["hopsworksUdf"]["statisticsArgumentNames"]).to eql(nil)

        end

      end
    end
  end
end