=begin
 This file is part of Hopsworks
 Copyright (C) 2024, Hopsworks AB. All rights reserved

 Hopsworks is free software: you can redistribute it and/or modify it under the terms of
 the GNU Affero General Public License as published by the Free Software Foundation,
 either version 3 of the License, or (at your option) any later version.

 Hopsworks is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Affero General Public License for more details.

 You should have received a copy of the GNU Affero General Public License along with this program.
 If not, see <https://www.gnu.org/licenses/>.
=end
module EnvironmentHelper

  def get_environments(project_id, query: nil, expected_status: 200, error_code: nil)
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments#{query}"
    expect_status_details(expected_status, error_code: error_code)
  end

  def get_environment(project_id, name, query: "", expected_status: 200, error_code: nil)
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{name}/#{query}"
    expect_status_details(expected_status, error_code: error_code) unless expected_status.nil?
  end

  def create_environment(project_id, env_name, base_image_name, description, expected_status: 201, error_code: nil)
    environment_conf = {
      "name": "#{env_name}",
      "description": "#{description}",
      "baseImage": {
        "name": "#{base_image_name}"
      }
    }
    post "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{env_name}", environment_conf
    expect_status_details(expected_status, error_code: error_code)
    get_project_env_by_id_and_name(project_id, env_name)
  end

  def get_project_env_by_id_and_name(id, name)
    Environment.find_by(project_id: "#{id}", name: "#{name}")
  end

  def get_project_envs_by_project_id(id)
    Environment.where(project_id: "#{id}")
  end

  def set_latest_environment_history_tag(project, env_name, tag)
    history = JSON.parse(get_environment_history(@project[:id], @custom_env_name))["items"][0]
    env_history = EnvironmentHistory.find_by(id: history[0][:id])
    env_history.docker_image = tag
    env_history.save()
  end

  def create_env(project, base_env_name, custom_env_name, wait_for_sync_complete=true, expected_status: 201, error_code: nil)
    env = get_project_env_by_id_and_name(project[:id], custom_env_name)
    project = get_project_by_name(project[:projectname])
    delete_env(project[:id], custom_env_name) unless env.nil?
    env = create_environment(project[:id], custom_env_name, base_env_name, "some description")
    expect_status_details(expected_status, error_code: error_code)
    if wait_for_sync_complete
      wait_for_sync(env)
    end
    env
  end

  def create_env_and_update_project(project, custom_env_name, base_env_name)
    env = get_project_env_by_id_and_name(project[:id], custom_env_name)
    project = get_project_by_name(project[:projectname])
    if env.nil?
      env = create_environment(project[:id], custom_env_name, base_env_name, "some description")
      wait_for_sync(env)
      get_project_env_by_id_and_name(project[:id], custom_env_name)
    else
      wait_for_sync(env)
      get_project_env_by_id_and_name(project[:id], custom_env_name)
    end
  end

  def export_env(project_id, name, expected_status: 200)
    post "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{name}?action=export"
    expect_status_details(expected_status)
  end

  def delete_env(project_id, name, expected_status: 204, error_code: nil)
    delete "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{name}"
    expect_status_details(expected_status, error_code: error_code)
    wait_for_running_command(project_id)
  end

  def wait_for_sync(env)
    wait_for do
      expect(CondaCommands.where(["environment_id = ? and op = ? and status = ?", env[:id], "CREATE", "FAILED"])).to be_empty
      CondaCommands.where(["environment_id = ? and op = ?", env[:id], "CREATE"]).empty?
    end
  end

  def list_envs(project_id)
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments"
  end

  def get_env(project_id, env, expected_status: 200)
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{env}"
    expect_status_details(expected_status)
  end

  def get_env_commands(project_id, env_name, expected_status: 200)
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{env_name}/commands"
    expect_status_details(expected_status)
  end

  def get_lib_commands(project_id, env_name, lib_name, expected_status: 200)
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{env_name}/libraries/#{lib_name}/commands"
    expect_status_details(expected_status)
  end

  def get_env_conflicts(project_id, env_name, query="", expected_status: 200)
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{env_name}/conflicts" + query
    expect_status_details(expected_status)
  end

  def get_environment_history(project_id, env_name, query="?sort_by=id:desc")
    get "#{ENV['HOPSWORKS_API']}/project/#{project_id}/python/environments/#{env_name}/history" + query
    expect_status_details(200)
  end
end
