=begin
 This file is part of Hopsworks
 Copyright (C) 2018, Logical Clocks AB. All rights reserved

 Hopsworks is free software: you can redistribute it and/or modify it under the terms of
 the GNU Affero General Public License as published by the Free Software Foundation,
 either version 3 of the License, or (at your option) any later version.

 Hopsworks is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 PURPOSE.  See the GNU Affero General Public License for more details.

 You should have received a copy of the GNU Affero General Public License along with this program.
 If not, see <https://www.gnu.org/licenses/>.
=end

require 'open3'

module HopsHelper

  @@hdfs_user = Variables.find_by(id: "hdfs_user").value
  @@rmyarn_user = Variables.find_by(id: "rmyarn_user").value
  @@yarn_user = Variables.find_by(id: "yarn_user").value
  @@hadoop_home = Variables.find_by(id: "hadoop_dir").value
  @@hopsworks_user = Variables.find_by(id: "hopsworks_user").value

  def hdfs_cmd(sub_cmd, cmd: "dfs")
    if ENV['KUBE_CLUSTER'] == "true"
      "hdfs #{cmd} #{sub_cmd}"
    else
      "sudo su #{@@hdfs_user} /bin/bash -c \"#{@@hadoop_home}/bin/hdfs #{cmd} #{sub_cmd}\""
    end
  end

  def run_hdfs_cmd(sub_cmd, cmd: "dfs")
    system hdfs_cmd(sub_cmd, cmd: cmd)
  end

  def chmod_hdfs(path, permission)
    run_hdfs_cmd("-chmod #{permission} #{path}")
    if $?.exitstatus > 0
      raise "Failed to chmod: #{permission} directory: #{path}"
    end
  end

  def mkdir(path, owner, group, mode)
    run_hdfs_cmd("-mkdir -p #{path}")
    if $?.exitstatus > 0
      raise "Failed to create directory: #{path}"
    end
    run_hdfs_cmd("-chown #{owner}:#{group} #{path}")
    if $?.exitstatus > 0
      raise "Failed to chown directory: #{path} to #{user}:#{group}"
    end
    run_hdfs_cmd("-chmod #{mode} #{path}")
    if $?.exitstatus > 0
      raise "Failed to chmod: #{mode} directory: #{path} "
    end
  end

  def copy_from_local(src, dest, owner, group, mode, name)
    run_hdfs_cmd("-copyFromLocal -f #{src} #{dest}")
    if $?.exitstatus > 0
      raise "Failed to copy: #{src} to #{dest} "
    end
    run_hdfs_cmd("-chown -R #{name}__#{owner}:#{group} #{dest}")
    if $?.exitstatus > 0
      raise "Failed to chown: #{dest} to #{owner}:#{group}"
    end
    run_hdfs_cmd("-chmod -R #{mode} #{dest}")
    if $?.exitstatus > 0
      raise "Failed to chmod: #{mode} directory: #{dest} "
    end
  end

  def test_dir(path)
    run_hdfs_cmd("-test -d #{path}")
    return $?.exitstatus == 0
  end

  def test_file(path)
    run_hdfs_cmd("-test -e #{path}")
    return $?.exitstatus == 0
  end

  def check_log_dir_has_files(application_id, project_name, job_type)
    cmd = hdfs_cmd("-count /Projects/#{project_name}/Logs/#{job_type}/#{application_id}")
    file_count = ""
    Open3.popen3(cmd) do |_, stdout, _, _|
      file_count = stdout.read
    end
    file_count.split(" ")[1].to_i > 0
  end

  def copy(src, dest, owner, group, mode, name)
    run_hdfs_cmd("-cp #{src} #{dest}")
    if $?.exitstatus > 0
      raise "Failed to copy directory: #{src} to #{dest}"
    end
    run_hdfs_cmd("-chown -R #{name}__#{owner}:#{group} #{dest}")
    if $?.exitstatus > 0
      raise "Failed to chown directory: #{dest} to #{owner}:#{group}"
    end
    run_hdfs_cmd("-chmod -R #{mode} #{dest}")
    if $?.exitstatus > 0
      raise "Failed to chmod: #{mode} directory: #{dest} "
    end
  end

  def rm(path)
    run_hdfs_cmd("-rm #{path}")
    if $?.exitstatus > 0
      raise "Failed to rm directory: #{path} "
    end
  end

  def rmdir(path)
    run_hdfs_cmd("-rm -r #{path}")
    if $?.exitstatus > 0
      raise "Failed to rm directory: #{path} "
    end
  end

  def get_storage_policy(path)
    cmd = hdfs_cmd("-getStoragePolicy -path #{path}", cmd: "storagepolicies")
    policy = ""
    Open3.popen3(cmd) do |_, stdout, _, _|
      policy = stdout.read
    end
    policy
  end

  def getHopsworksUser
    @@hopsworks_user
  end

  def touchz(path, owner, group)
    run_hdfs_cmd("-touchz #{path}")
    if $?.exitstatus > 0
      raise "Failed to create file: #{path}"
    end
    run_hdfs_cmd("-chown #{owner}:#{group} #{path}")
    if $?.exitstatus > 0
      raise "Failed to change owner: #{path} to #{owner}:#{group}"
    end
  end

  ################################################### YARN helpers #####################################################

  def get_application_state(app_id, states)
    if ENV['KUBE_CLUSTER'] == "true"
      raise "Operation not supported in a kube cluster"
    end
    cmd = "sudo su #{@@rmyarn_user} /bin/bash -c \"#{@@hadoop_home}/bin/yarn app -list -appStates #{states} | grep #{app_id}\""
    state = ""
    Open3.popen3(cmd) do |_, stdout, _, _|
      state = stdout.read
    end
    # state looks like
    # application_1600882170037_0008	      demo_job_5_jar	      Hopsworks-Yarn	demo_spark_deb57f91__deb57f97	   default	            KILLED	            KILLED	           100%	http://resourcemanager.service.consul:8088/cluster/app/application_1600882170037_0008
    state.split[5]
  end

  ######################################################################################################################

end