# This file is part of Hopsworks
# Copyright (C) 2024, Hopsworks AB. All rights reserved
#
# Hopsworks is free software: you can redistribute it and/or modify it under the terms of
# the GNU Affero General Public License as published by the Free Software Foundation,
# either version 3 of the License, or (at your option) any later version.
#
# Hopsworks is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
# without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License along with this program.
# If not, see <https://www.gnu.org/licenses/>.
#

require 'json'

describe "On #{ENV['OS']}" do
  after(:all) { clean_all_test_projects(spec: "transformations") }

  describe "transformation" do
    describe "internal" do
      context 'with valid project, featurestore service enabled' do
        before :all do
          with_valid_project
        end

        it "should be able to retrieve built-in transformation registered during project creation" do
            featurestore_id = get_featurestore_id(@project.id)
            endpoint = "#{ENV['HOPSWORKS_API']}/project/#{@project
            .id}/featurestores/#{featurestore_id}/transformationfunctions"
            json_result = get endpoint
            expect_status_details(200)
            parsed_json = JSON.parse(json_result)
  
            feature_transformation = parsed_json["items"].select{ |f| f["hopsworksUdf"]["name"] == "min_max_scaler"}.first
            expect(feature_transformation["hopsworksUdf"]["name"]).to eql("min_max_scaler")
            expect(feature_transformation["hopsworksUdf"]["outputTypes"]).to eql(["double"])
            expect(feature_transformation["hopsworksUdf"]["transformationFunctionArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["statisticsArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["droppedArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["transformationFeatures"]).to eql([])
  
            feature_transformation = parsed_json["items"].select{ |f| f["hopsworksUdf"]["name"] == "standard_scaler"}.first
            expect(feature_transformation["hopsworksUdf"]["name"]).to eql("standard_scaler")
            expect(feature_transformation["hopsworksUdf"]["outputTypes"]).to eql(["double"])
            expect(feature_transformation["hopsworksUdf"]["transformationFunctionArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["statisticsArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["droppedArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["transformationFeatures"]).to eql([])
  
            feature_transformation = parsed_json["items"].select{ |f| f["hopsworksUdf"]["name"] == "robust_scaler"}.first
            expect(feature_transformation["hopsworksUdf"]["name"]).to eql("robust_scaler")
            expect(feature_transformation["hopsworksUdf"]["outputTypes"]).to eql(["double"])
            expect(feature_transformation["hopsworksUdf"]["transformationFunctionArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["statisticsArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["droppedArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["transformationFeatures"]).to eql([])
  
            feature_transformation = parsed_json["items"].select{ |f| f["hopsworksUdf"]["name"] == "label_encoder"}.first
            expect(feature_transformation["hopsworksUdf"]["name"]).to eql("label_encoder")
            expect(feature_transformation["hopsworksUdf"]["outputTypes"]).to eql(["bigint"])
            expect(feature_transformation["hopsworksUdf"]["transformationFunctionArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["statisticsArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["droppedArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["transformationFeatures"]).to eql([])

            feature_transformation = parsed_json["items"].select{ |f| f["hopsworksUdf"]["name"] == "one_hot_encoder"}.first
            expect(feature_transformation["hopsworksUdf"]["name"]).to eql("one_hot_encoder")
            expect(feature_transformation["hopsworksUdf"]["outputTypes"]).to eql(["boolean"])
            expect(feature_transformation["hopsworksUdf"]["transformationFunctionArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["statisticsArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["droppedArgumentNames"]).to eql(["feature"])
            expect(feature_transformation["hopsworksUdf"]["transformationFeatures"]).to eql([])
          end

          it "should be able to create and then delete transformation function" do
            featurestore_id = get_featurestore_id(@project.id)
            json_result = register_transformation_fn(@project.id, featurestore_id)
            parsed_json = JSON.parse(json_result)
            expect_status_details(200)
            transformation_function_id = parsed_json["id"]
            endpoint = "#{ENV['HOPSWORKS_API']}/project/#{@project.id}/featurestores/#{featurestore_id}/transformationfunctions/#{transformation_function_id}"
            delete endpoint
            expect_status_details(200)
          end
      end
    end
  end
end